@@ -227,6 +227,7 @@ setClass("item_GR",
 #' @slot se a matrix containing item parameter standard errors.
 #' @slot raw the raw input \code{\link{data.frame}} used in \code{\link{loadItemPool}} to create this object.
 #' @slot raw_se the raw input \code{\link{data.frame}} used in \code{\link{loadItemPool}} to create this object.
+#' @slot unique whether item IDs must be unique for this object to be a valid object.
 #'
 #' @export
 setClass("item_pool",
@@ -241,7 +242,8 @@ setClass("item_pool",
     ipar    = "matrix",
     se      = "matrix",
     raw     = "data.frame",
-    raw_se  = "dataframe_or_null"
+    raw_se  = "dataframe_or_null",
+    unique  = "logical"
   ),
   prototype = list(
     ni      = numeric(0),
@@ -254,14 +256,17 @@ setClass("item_pool",
     ipar    = matrix(0),
     se      = matrix(0),
     raw     = data.frame(),
-    raw_se  = NULL
+    raw_se  = NULL,
+    unique  = logical(0)
   ),
   validity = function(object) {
-    if (length(unique(object@id)) != length(object@id)) {
-      stop("Entries in @id must be unique")
+    if (object@unique) {
+      if (length(unique(object@id)) != length(object@id)) {
+        stop("item_pool: entries in @id must be unique if @unique == TRUE")
+      }
     }
     if (dim(object@raw)[1] != object@ni) {
-      stop("Number of items in @raw does not match @ni. Change @ni to match @raw.")
+      stop("item_pool: number of items in @raw does not match @ni")
     }
     return(TRUE)
   }
